@@ -51,6 +51,7 @@ class Document:
             doc_path: str = "text",
             json_doc: dict = None,
             analyzers: list = [],
+            error: str = "",
     ):
 
         self.original_text = original_text
@@ -58,6 +59,7 @@ class Document:
         self.analyzers = analyzers
         self.dominant_language_code = dominant_language_code
         self.analysis_lang = analysis_lang
+        self.error = error
         self.json_doc = json_doc
 
         self.entity_mapper = entity_mapper
@@ -98,9 +100,20 @@ class Document:
                 self.add_fact(fact)
 
 
+    @staticmethod
+    def handle_null_values_in_facts(facts: List[dict]):
+        container = []
+        if facts:
+            for fact in facts:
+                new_fact = {key: value for key, value in fact.items() if value is not None}
+                container.append(new_fact)
+        return container
+
+
     @staticmethod
     def remove_duplicate_facts(facts: List[dict]):
         if facts:
+            facts = Document.handle_null_values_in_facts(facts)
             set_of_jsons = {json.dumps(fact, sort_keys=True, ensure_ascii=False) for fact in facts}
             without_duplicates = [json.loads(unique_fact) for unique_fact in set_of_jsons]
             return without_duplicates
@@ -155,6 +168,8 @@ class Document:
             "detected": self.dominant_language_code,
             "analysis": self.analysis_lang
         }
+        if self.error:
+            container["error"] = self.error
 
         if "lemmas" in self.analyzers:
             container["lemmas"] = self.get_lemma()
