@@ -26,10 +26,12 @@ class Fact:
 
 
     def to_json(self):
-        return {
+        container = {
             "str_val": self.fact_value,
             "spans": json.dumps(self.spans),
             "fact": self.fact_type,
             "doc_path": self.doc_path,
-            "lemma": self.fact_lemma
         }
+        if self.fact_lemma:
+            container["lemma"] = self.fact_lemma
+        return container
