@@ -15,6 +15,7 @@ from pelecanus import PelicanJson
 from texta_mlp.document import Document
 from texta_mlp.entity_mapper import EntityMapper
 from texta_mlp.utils import parse_bool_env
+from texta_mlp.exceptions import LanguageNotSupported
 
 
 # Languages supported by default.
@@ -79,7 +80,6 @@ class MLP:
         self.default_lang = default_language_code
         self.use_default_lang = use_default_language_code
         self.resource_dir = resource_dir
-        self.use_gpu = use_gpu
 
         self.resource_dir_pathlib = pathlib.Path(resource_dir)
         self.not_entities_path = self.resource_dir_pathlib / "concatenator" / "not_entities.txt"
@@ -88,7 +88,13 @@ class MLP:
 
         self.prepare_resources(refresh_data)
 
-        self.stanza_pipelines = self._load_stanza_pipelines(logging_level)
+        # This is for CUDA OOM exceptions. Fall back to CPU if needed.
+        try:
+            self.stanza_pipelines = self._load_stanza_pipelines(logging_level, use_gpu)
+        except RuntimeError:
+            # Try loading using CPU
+            self.stanza_pipelines = self._load_stanza_pipelines(logging_level, False)
+
         self.entity_mapper = self._load_entity_mapper()
         self.loaded_entity_files = []
 
@@ -202,10 +208,10 @@ class MLP:
         '''
         if lang not in self.supported_langs:
             analysis_lang = self.default_lang
-            sentences, entities = self._get_stanza_tokens(analysis_lang, processed_text) if processed_text else ([], [])
+            sentences, entities, e = self._get_stanza_tokens(analysis_lang, processed_text) if processed_text else ([], [], "")
         else:
             analysis_lang = lang
-            sentences, entities = self._get_stanza_tokens(analysis_lang, processed_text) if processed_text else ([], [])
+            sentences, entities, e = self._get_stanza_tokens(analysis_lang, processed_text) if processed_text else ([], [], "")
 
         document = Document(
             original_text=processed_text,
@@ -217,7 +223,8 @@ class MLP:
             json_doc=json_object,
             doc_path=doc_paths,
             entity_mapper=self.entity_mapper,
-            concat_resources=self.concat_resources
+            concat_resources=self.concat_resources,
+            error=e
         )
         return document
 
@@ -230,19 +237,29 @@ class MLP:
 
 
     def _get_stanza_tokens(self, lang: str, raw_text: str):
-        pipeline = self.stanza_pipelines[lang](raw_text)
-
         sentences = []
         entities = []
-        pip_pat = re.compile(r"(?<=\d)_(?=\d)")
-        for sentence in pipeline.sentences:
-            words = []
-            for word in sentence.words:
-                words.append(word)
-            sentences.append(words)
-            for entity in sentence.entities:
-                entities.append(entity)
-        return sentences, entities
+        e = ""
+        try:
+            pipeline = self.stanza_pipelines[lang](raw_text)
+
+            pip_pat = re.compile(r"(?<=\d)_(?=\d)")
+            for sentence in pipeline.sentences:
+                words = []
+                for word in sentence.words:
+                    words.append(word)
+                sentences.append(words)
+                for entity in sentence.entities:
+                    entities.append(entity)
+
+        except KeyError as e:
+            raise LanguageNotSupported(f"Language {lang} not supported. Check the list of supported languages.")
+
+        except Exception as e:
+            self.logger.exception(e)
+            return sentences, entities, repr(e)
+
+        return sentences, entities, e
 
 
     def _get_stanza_ner(self, lang: str, raw_text: str):
@@ -250,7 +267,7 @@ class MLP:
         return [entity for sentence in pipeline.sentences for entity in sentence.entities]
 
 
-    def _load_stanza_pipelines(self, logging_level):
+    def _load_stanza_pipelines(self, logging_level, use_gpu):
         """
         Initializes Stanza Pipeline objects all at once to save time later.
         """
@@ -261,7 +278,7 @@ class MLP:
                 lang=lang,
                 dir=str(stanza_resource_path),
                 processors=self._get_stanza_processors(lang),
-                use_gpu=self.use_gpu,
+                use_gpu=use_gpu,
                 logging_level=logging_level
             )
         return stanza_pipelines
@@ -272,7 +289,6 @@ class MLP:
         """
         Returns processor options based on language and NER support in Stanza.
         """
-        print(lang)
         if lang in STANZA_NER_SUPPORT:
             return "tokenize,pos,lemma,ner"
         else:
